/*++

Copyright (c) 2001 Compaq Computer Corporation

Module Name:

    CpqCicpu.c

Abstract:

    This module contains the IA32 binary data region containing
    the code for a 'C'-style call to perform ATOMIC operations.

Author:

    Thomas J. Bonola (Tom.Bonola@compaq.com) Mar-2001

Environment:

    OS Agnostic
    IA32 processors.

Compiler Settings:

    This module requires the following compiler settings.

        *   1-byte packed data structures
        *   All functions must adhere to standard 'C' calling convention
        
Revision History:

--*/

//*******************************************************************
//*                    INTEL ARCHITECTURE CENTRIC
//* The following static data area contains Intel specific ia32
//* code to execute the CpqCiCas() routine.  Given the latest 
//* compiler experience in trying to comprehend inline assembler,
//* it is more prudent to hand-assemble the Intel based opcodes 
//* into a static byte array and map into a 'C'-style function call
//* so that we do not require another assembler module to build this
//* stuff.
//*
//*    int CpqCiCas( vu64_t * Memory, u64_t Expected, u64_t New );
//*
//*******************************************************************

#include "CpqCiTyp.h"
#include "CpqCi.h"

int CpqCiCas( vu64_t * Memory, u64_t Expected, u64_t New )
{
        int rc;

        asm volatile (
                     "lock cmpxchg8b (%1)\r\n"
                     "jnz 1f\r\n"
                     "mov $0x1, %0\r\n"
                     "jmp 2f\r\n"
                     "1: mov $0x0, %0\r\n"
                     "2: nop\r\n"
                     :"=r" (rc)
                     :"D" (Memory), "a" ((u32_t)Expected), "d" ((u32_t)(Expected>>32)), "b" ((u32_t)New), "c" ((u32_t)(New>>32)));
        return rc;
}


